#!/usr/bin/perl

use strict;
use warnings;

use Data::Dumper;

my $marksPT = load_marks("Markers.PT");
my $marksEN = load_marks("Markers.EN");

chomp(my $PT = lc(<>));
chomp(my $EN = lc(<>));

my $pt_tree = split_by_marks($marksPT, $PT);
my $en_tree = split_by_marks($marksEN, $EN);

$pt_tree = combine_marks($pt_tree);
$en_tree = combine_marks($en_tree);



sub combine_marks {
  my $list = shift;
  my $previous = "";
  my $done = [];
  for my $item (@$list) {
    if ($item->[0] eq '[MARKS]') {
      $previous = join(" ", @$item[1..$#$item]);
    } else {
      push @$done, join(" ",$previous,@$item);
      $previous = "";
    }
  }
  push @$done, $previous   if ($previous);
  return $done;
}


sub split_by_marks {
  my ($marks, $string) = @_;
  my @string = split /\s+/, $string;

  my $word;
  my $nS = [];
  my @S;
  while ($word = shift @string) {
    my $length = 2 + $#string;

    my $found = 0;
    my $cmark;
    for my $mark (@$marks) {
      my $mlength = scalar(@$mark);
      next if $mlength > $length;

      if ($word eq $mark->[0]) {
        $found = 1;
        for my $i (1..$mlength-1) {
          $found = 0 if $string[$i-1] ne $mark->[$i]
        }
        if ($found) {
          $cmark = $mark;
          last;
        }
      }
    }

    if ($found) {
      my $mlength = scalar(@$cmark) - 1;
      my @x = ($word);
      push @x, shift @string while $mlength--;
      if (@$nS) {
        push @S, $nS;
        $nS = [];
        unshift @x, '[MARKS]';
        push @S, \@x;
      } else {
        unshift @x, '[MARKS]' unless @S;
        # get last and add
        my $lastMark = pop @S;
        push @$lastMark, @x;
        push @S, $lastMark;
      }
    } else {
      push @$nS, $word;
    }
  }
  push @S, $nS if (@$nS);

  return \@S;
}

sub load_marks {
  my $marks_file = shift;
  my $marks;

  open M, $marks_file or die "Can not open file $marks_file: $!";
  while (<M>) {
    chomp;
    push @$marks, $_;
  }

  $marks = [sort {length($b) <=> length($a)} @$marks];

  return [map { [split /\s+/] } @$marks]
}
