#!/usr/bin/perl -w

use Test::More qw/no_plan/;

print STDERR "Warning: some of these tests can take about 1 minute on slow machines\n";

our($DIC1,$DIC2) = (undef,undef);

# 1
ok(-f "input/EN-tok", "Testing English corpus existence");

# 2
ok(-f "input/PT-tok", "Testing Portuguese (BR) corpus existence");

# Following code will overlap with nat-these code.
# This "must" be done so tests can be done at each step.

###
### NAT-PRE
###
# 3 a 8
my @prefiles = qw/PT.lex EN.lex PT.crp PT.crp.index EN.crp.index EN.crp/;
for (@prefiles) {
  unlink if -f;
  ok(! -f, "Checking if file $_ does no exist");
}

`../src/nat-pre -q input/PT-tok input/EN-tok PT.lex EN.lex PT.crp EN.crp`;

# 9 a 14
for (@prefiles) {
  ok(-f, "Checking if file $_ exists");
}

# 15
ok(-s "PT.crp.index" == -s "EN.crp.index", "Index files have the same size");

open (P, "../src/nat-css PT.lex PT.crp EN.lex PT.crp all|") or die "Cannot open ../src/nat-css as pipe";
my @corpus = <P>;
close P;

like($corpus[12], qr/^Diante das/, "Index 1 is what we hope it is");
like($corpus[16], qr/^O planejamento/, "Index 2 is what we hope it is");
like($corpus[24], qr/^Este trabalho/, "Index 5 is what we hope it is");

###
### nat-initmat
###
my @initmatfiles = qw/PT-EN.mat/;
for (@initmatfiles) {
  unlink if -f;
  ok(! -f, "Checking if file $_ was correctly deleted");
}
`../src/nat-initmat PT.crp EN.crp PT-EN.mat 2>/dev/null`;
die "Failed 'nat-initmat PT.crp EN.crp PT-EN.mat'\n" if $?;

for (@initmatfiles) {
  ok(-f, "Checking if file $_ exists");
}

###
### nat-ipfp
###
my @ipfpfiles = qw/PT-EN.ipfp.mat/;
for (@ipfpfiles) {
  unlink if -f;
  ok(! -f, "Checking if file $_ was correctly deleted");
}
`../src/nat-ipfp 3 PT.crp EN.crp PT-EN.mat PT-EN.ipfp.mat 2>/dev/null`;
for (@ipfpfiles) {
  ok(-f, "Checking if file $_ exists");
}

###
### nat-mat2dic   (ipfp.mat ipfp.dic)
###
my @mat2dicfiles = qw/PT-EN.ipfp.dic/;
for (@mat2dicfiles) {
  unlink if -f;
  ok(! -f, "Checking if file $_ was correctly deleted");
}
`../src/nat-mat2dic PT-EN.ipfp.mat PT-EN.ipfp.dic 2>/dev/null`;
for (@mat2dicfiles) {
  ok(-f, "Checking if file $_ exists");
}

###
### nat-postbin (PT-EN.ipfp.dic PT.lex EN.lex PT-EN.pl EN-PT.pl)
###
my @postfiles = qw/EN-PT.bin PT-EN.bin/;
for (@postfiles) {
  unlink if -f;
  ok(! -f, "Checking if file $_ was correctly deleted");
}
`../src/nat-postbin PT-EN.ipfp.dic PT.crp.partials EN.crp.partials PT.lex EN.lex PT-EN.bin EN-PT.bin`;
for (@postfiles) {
  ok(-f, "Checking if file $_ exists");
}

###
### nat-dumpDict
###
my @DumpDict = qw/PT-EN.txt/;
for (@DumpDict) {
  unlink if -f;
  ok(! -f, "Checking if file $_ still does not exist");
}
`perl -Mblib=../NAT/blib ../NAT/scripts/nat-dumpDicts -full PT.lex PT-EN.bin EN.lex EN-PT.bin > PT-EN.txt`;
for (@DumpDict) {
  ok(-f, "Checking if file $_ exist");
}

###
###
###
eval{ require "PT-EN.txt" };
ok(!$@, "NATools output does not have errors");

ok(defined($DIC1));
ok(defined($DIC2));

{
  my $ORIGINAL = do "input/PT-tok.wc";
  for (keys %$ORIGINAL) {
    next if $_ eq '$';
    print STDERR "\n\n$_\n\n" unless $ORIGINAL->{$_} == $DIC1->{$_}{count};
    is($DIC1->{$_}{count}, $ORIGINAL->{$_}, "Testing word '$_'");
  }

}


{
  my $ORIGINAL = do "input/EN-tok.wc";
  for (keys %$ORIGINAL) {
    next if $_ eq '$';
    print STDERR "\n\n$_\n\n" unless $ORIGINAL->{$_} == $DIC2->{$_}{count};
    is($DIC2->{$_}{count}, $ORIGINAL->{$_}, "Testing word '$_'");
  }

}

unlink(@prefiles,@initmatfiles,@ipfpfiles,@mat2dicfiles,@postfiles);


### EOF
